class FilterWebpackOutput {
  constructor(rules) {

    if (this.isRegExp(rules)) {
      this.rules = [rules];
    }

    if (this.isValidArray(rules)) {
      this.rules = [ ...rules ];
    }

    if (!this.rules) {
      throw new Error('Param should be a RegExp or array of RegExp');
    }
  }

  isRegExp(rule) {

    return rule && rule instanceof RegExp;
  }

  isValidArray(arr) {

    return Array.isArray(arr) && arr.length > 0 && arr.every(this.isRegExp);
  }

  isSkipped(file) {

    return rule => rule.test(file);
  }

  skippedFiles(file) {
    
    return this.rules.some(this.isSkipped(file));
  }
 
  skipFile(assets) {
    
    return file => delete assets[file]; 
  }

  filterFiles({ assets }) {

    Object.keys(assets)
      .filter(this.skippedFiles.bind(this))
      .forEach(this.skipFile(assets));
  }

  apply(compiler) {
    compiler.hooks.emit.tap('FilterWebpackOutput', this.filterFiles.bind(this));
  }
}

module.exports = FilterWebpackOutput;
